%% Understanding covariance matrix.

load fisheriris;                        % We will work with the Iris dataset.
X = meas;                               % [Sepal length, Sepal width, Petal length, Petal width]

C = cov( X );

% Indexes.
sL = 1;
sW = 2;
pL = 3;
pW = 4;

%% Positive covariance.

figure;
plot( X(:,pL), X(:,pW), 'o', 'Color', [0, 0.5, 0.5] );
xlabel( 'Petal length' );
ylabel( 'Petal width' );
title( sprintf( 'Positive covariance of %f', C(pL,pW) ) );
axis equal;
hold on;

% Fit a line to data.
A = [X(:,pL) X(:,pL).^0];
c = pinv(A) * X(:,pW);
plot( X(:,pL), A*c, 'm.' );

mPL = mean( X(:,pL) );                  % Means of petal length and petal width.
mPW = mean( X(:,pW) );
plot( mPL, mPW, 'go', 'MarkerFaceColor', 'b', 'MarkerSize', 10 );

legend( 'Iris dataset', ...
    sprintf( '(%.6f)x + (%.6f) = y', c(1), c(2) ), ...
    sprintf( 'Means (%.6f, %.6f)', mPL, mPW ) );
hold off;                               % Check: c(1) = cov(pL,pW)/var(pL).

%% Negative covariance.

figure;
plot( X(:,pL), X(:,sW), 'o', 'Color', [0, 0.5, 0.5] );
xlabel( 'Petal length' );
ylabel( 'Sepal width' );
title( sprintf( 'Negative covariance of %f', C(pL,sW) ) );
axis equal;
hold on;

% Fit a line to data.
A = [X(:,pL) X(:,pL).^0];
c = pinv(A) * X(:,sW);
plot( X(:,pL), A*c, 'm.' );

mPL = mean( X(:,pL) );                  % Means of petal length and sepal width.
mSW = mean( X(:,sW) );
plot( mPL, mSW, 'go', 'MarkerFaceColor', 'b', 'MarkerSize', 10 );

legend( 'Iris dataset', ...
    sprintf( '(%.6f)x + (%.6f) = y', c(1), c(2) ), ...
    sprintf( 'Means (%.6f, %.6f)', mPL, mSW ) );
hold off;                               % Check: c(1) = cov(pL,sW)/var(pL).

%% No covariance.

x = linspace(-2,2,20)';
y = x.^2;
D = [x y];
C = cov(D);
figure;
plot( x, y, 'mo-', 'MarkerFaceColor', [0,0.5,0.5] );
xlabel( 'x' );
ylabel( 'y' );
title( sprintf( 'Covariance betwen x and y: %f', C(1,2) ) );
legend( 'y = x.^2' );
axis equal;
axis( [ min(x)-1 max(x)+1 min(y)-1 max(y)+1 ] );
